@@ -23,7 +23,6 @@
  *   + Documents
  *   + Eye of GNOME
  *   + Shotwell
- *   + Totem
  */
 
 
@@ -83,64 +82,29 @@ enum
 };
 
 
-static void
-photos_utils_put_pixel (guchar *p)
-{
-  p[0] = 46;
-  p[1] = 52;
-  p[2] = 54;
-  p[3] = 0xff;
-}
-
-
-void
-photos_utils_border_pixbuf (GdkPixbuf *pixbuf)
-{
-  gint height;
-  gint width;
-  gint rowstride;
-  gint x;
-  gint y;
-  guchar *pixels;
-
-  height = gdk_pixbuf_get_height (pixbuf);
-  width = gdk_pixbuf_get_width (pixbuf);
-
-  pixels = gdk_pixbuf_get_pixels (pixbuf);
-  rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-
-  /* top */
-  for (x = 0; x < width; x++)
-    photos_utils_put_pixel (pixels + x * 4);
-
-  /* bottom */
-  for (x = 0; x < width; x++)
-    photos_utils_put_pixel (pixels + (height - 1) * rowstride + x * 4);
-
-  /* left */
-  for (y = 1; y < height - 1; y++)
-    photos_utils_put_pixel (pixels + y * rowstride);
-
-  /* right */
-  for (y = 1; y < height - 1; y++)
-    photos_utils_put_pixel (pixels + y * rowstride + (width - 1) * 4);
-}
-
-
 GdkPixbuf *
 photos_utils_center_pixbuf (GdkPixbuf *pixbuf, gint size)
 {
   GdkPixbuf *ret_val;
   gint height;
+  gint pixbuf_size;
   gint width;
 
-  ret_val = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, size, size);
-  gdk_pixbuf_fill (ret_val, 0x00000000);
-
   height = gdk_pixbuf_get_height (pixbuf);
   width = gdk_pixbuf_get_width (pixbuf);
+
+  pixbuf_size = MAX (height, width);
+  if (pixbuf_size >= size)
+    {
+      ret_val = g_object_ref (pixbuf);
+      goto out;
+    }
+
+  ret_val = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, size, size);
+  gdk_pixbuf_fill (ret_val, 0x00000000);
   gdk_pixbuf_copy_area (pixbuf, 0, 0, width, height, ret_val, (size - width) / 2, (size - height) / 2);
 
+ out:
   return ret_val;
 }
 
@@ -306,7 +270,6 @@ photos_utils_create_placeholder_icon_for_scale (const gchar *name, gint size, gi
 
   size_scaled = size * scale;
   centered_pixbuf = photos_utils_center_pixbuf (pixbuf, size_scaled);
-  photos_utils_border_pixbuf (centered_pixbuf);
 
   ret_val = centered_pixbuf;
   centered_pixbuf = NULL;
@@ -533,7 +496,15 @@ photos_utils_downscale_pixbuf_for_scale (GdkPixbuf *pixbuf, gint size, gint scal
   scaled_size = size * scale;
 
   /* On Hi-Dpi displays, a pixbuf should never appear smaller than on
-   * Lo-Dpi. Therefore, if a pixbuf lies between (size, size * scale)
+   * Lo-Dpi.
+   *
+   * Sometimes, a pixbuf can be slightly smaller than size. eg.,
+   * server-generated thumbnails for remote tems. Scaling them up
+   * won't cause any discernible loss of quality and will make our
+   * letterboxed grid look nicer. 75% of 'scale' has been chosen as
+   * the arbitrary definition of 'slightly smaller'.
+   *
+   * Therefore, if a pixbuf lies between (3 * size / 4, size * scale)
    * we scale it up to size * scale, so that it doesn't look smaller.
    * Similarly, if a pixbuf is smaller than size, then we increase its
    * dimensions by the scale factor.
@@ -543,7 +514,7 @@ photos_utils_downscale_pixbuf_for_scale (GdkPixbuf *pixbuf, gint size, gint scal
     {
       ret_val = g_object_ref (pixbuf);
     }
-  else if (pixbuf_size > size)
+  else if (pixbuf_size > 3 * size / 4)
     {
       if (height == width)
         {
@@ -1168,6 +1139,21 @@ photos_utils_get_urns_from_items (GList *items)
 }
 
 
+const gchar *
+photos_utils_get_version (void)
+{
+  const gchar *ret_val = NULL;
+
+#ifdef PACKAGE_COMMIT_ID
+  ret_val = PACKAGE_COMMIT_ID;
+#else
+  ret_val = PACKAGE_VERSION;
+#endif
+
+  return ret_val;
+}
+
+
 GIcon *
 photos_utils_icon_from_rdf_type (const gchar *type)
 {
